
// Echo.c implementation file
//---------------------------
#include <stdio.h>
#include <string.h>
#include "echo.h"
#include "ascii.h"
#include "defines.h"
#include "uart.h"
#include "crc16.h"
#include "timer.h"

#define CMD_IDLE	0x00
#define CMD_FLAG	0x01
#define CMD_CRC16	0x02

typedef enum tag_ECHO_STATE
//============================
{
	ECHO_idle=0,
	ECHO_data,
	ECHO_flagged
}
ECHO_STATE;

static INITIALIZE* g_pinitialize;
static WORD g_crc16;

static int echo_transmit(const int cmd, const char data)
//=========================================================
{
	int error_code;
	if (cmd!=CMD_IDLE)
	{
		g_crc16=crc16_onbyte(g_crc16,data);
	}
	error_code=uart_transmit(&data,1);
	if (error_code!=OKAY)
	{
		return error_code;
	}
	if (((cmd&CMD_FLAG)!=0) && (data==stx))
	{
		error_code=echo_transmit(CMD_IDLE,stx);
		if (error_code!=OKAY)
		{
			return error_code;
		}
	}
	if ((cmd&CMD_CRC16)!=0)
	{
		error_code=echo_transmit(CMD_IDLE,HIBYTE(g_crc16));
		if (error_code!=OKAY)
		{
			return error_code;
		}
		error_code=echo_transmit(CMD_IDLE,LOBYTE(g_crc16));
		if (error_code!=OKAY)
		{
			return error_code;
		}
	}
	return OKAY;
}

static int echo_receive(char* pdata)
//====================================
{
	ECHO_STATE echo_state;
	echo_state=ECHO_idle;
	char string[64];
	char* pdestination;
	int error_code;
	int length;
	int index;
	int received;
	int timeout;
	echo_state=ECHO_idle;
	timeout=timer_set(g_pinitialize->timeout);
	if (timeout==ERROR)
	{
		return ERROR;
	}
	while (timer_passed(timeout)==OKAY)
	{
		error_code=uart_receive(string,&length);
		if (error_code==OKAY && length>0)
		{
			index=0;
			while (index<length)
			{
				switch (echo_state)
				{
					case ECHO_idle:
					{
						if (string[index++]==stx)
						{
							received=0;
							pdestination=pdata;
							g_crc16=START_VALUE_ECHO;
							echo_state=ECHO_data;
						}
						break;
					}
					case ECHO_data:
					{
						*pdestination=string[index++];
						g_crc16=crc16_onbyte(g_crc16,*pdestination);
						if (*pdestination==stx)
						{
							echo_state=ECHO_flagged;
						}
						pdestination++;
						received++;
						if (received>1)
						{
							if (received==pdata[1]+4)
							{
								timer_unset(timeout);
								*(pdestination-2)=nul;
								if (g_crc16==0)
								{
									return VALID;
								}
								else
								{
									return ERROR;
								}
							}
						}
						break;
					}
					case ECHO_flagged:
					{
						if (string[index]==stx)
						{
							index++;
						}
						else
						{
							received=0;
							pdestination=pdata;
							g_crc16=START_VALUE_ECHO;
						}
						echo_state=ECHO_data;
						break;
					}
				}
			}
		}
	}
	timer_unset(timeout);
	return error_code;
}

int echo_initialize(INITIALIZE* pinitialize)
//============================================
{
	int error_code;
	g_pinitialize=pinitialize;
	error_code=uart_open(pinitialize->pdevice);	
	if (error_code!=OKAY)
	{
		return error_code;
	}
	error_code=crc16_initialize(POLYNOM_CCITT);	
	if (error_code!=OKAY)
	{
		return error_code;
	}
	error_code=timer_init();
	if (error_code!=OKAY)
	{
		return error_code;
	}
	return OKAY;
}

int echo_terminate(void)
//=======================
{
	int error_code;
	error_code=timer_done();
	if (error_code!=OKAY)
	{
		return error_code;
	}
	error_code=uart_close();
	if (error_code!=OKAY)
	{
		return error_code;
	}
	return OKAY;
}

int echo_command(int device, BOOL response, char* pdata)
//========================================================
{
	int cmd;
	int index;
	int error_code;
	char frame[64];
	g_crc16=START_VALUE_ECHO;
	frame[0]=stx;
	frame[1]=device;
	frame[2]=strlen(pdata);
	strcpy(frame+3,pdata);
	for (index=0; index<(frame[2]+3); index++)
	{
		cmd=CMD_IDLE;
		if (index>0)
		{
			cmd|=CMD_FLAG;
		}
		if (index==(frame[2]+2))
		{ 
			cmd|=CMD_CRC16;
		}
		error_code=echo_transmit(cmd,frame[index]);
		if (error_code!=OKAY)
		{
			return error_code;
		}
	}
	if (response)
	{
		return echo_receive(pdata);
	}
	return error_code;
}

